

set -ex



conda-repo --help
conda repo config --show
conda repo config --set sites.test.url http://localhost:8088/api/
conda repo config --set default_site test
conda repo config --show
pip check
exit 0
